-- My own rendition of the Drop Dash! I know this has been done before but shush

-- Freeslot and define what our states and sounds are
-- Drop Dash sprites by DirkTH and dust from Drop Dash Plus by MegAmi
freeslot("SPR_DRPD", "SPR_DPDU", "S_DROP_DASH", "S_DROP_DUST", "MT_DROP_DUST", "sfx_drpdsh")

states[S_DROP_DASH] = {
	sprite = SPR_DRPD,
	frame = FF_ANIMATE,
	tics = -1,
	var1 = 5,
	var2 = 1,
	nextstate = S_DROP_DASH
}

states[S_DROP_DUST] = {
	sprite = SPR_DPDU,
	frame = FF_ANIMATE|FF_PAPERSPRITE,
	tics = -1,
	var1 = 7,
	var2 = 2
}

mobjinfo[MT_DROP_DUST] = {
	doomednum = -1,
	spawnstate = S_DROP_DUST,
	spawnhealth = 9999,
	radius = 32*FU,
	flags = MF_NOGRAVITY
}

sfxinfo[sfx_drpdsh].caption = "Drop Dash"

-- A prerequisite needed for the Drop Dash dust
-- For some odd reason, it wouldn't disappear
addHook("MobjThinker", function(mobj)
    if mobj and mobj.valid
	and mobj.type == MT_DROP_DUST
    and not mobj.customTimer then
        mobj.customTimer = $ + 1
        if mobj.customTimer == 8 then
            P_RemoveMobj(mobj)
			P_KillMobj(mobj)
        end
    end
end, MT_DROP_DUST)

-- Variables and the dust function we need before going to code this
-- Dust function from Generations Style Drift by TGTLS
-- Thank MainMemory for the thrust value logic, since it came from their Sonic Retro post
local DROPSKIN = "sonic"
local DROPTHRUST = 55*FRACUNIT

addHook("PlayerThink", function(p)
	p.mo.dropdash = $ or false
	p.mo.droptime = $ or 0
	p.mo.lastz = $ or p.mo.z
	p.mo.spindown = p.pflags & PF_SPINDOWN
end)

rawset(_G, "DD_DoDust", function(p)
	local mo = p.mo
	for i = -1, 1, 2
		local ang = i * ANG30
		local factor = FixedMul(4*mo.scale, cos(ang))
		local x = P_ReturnThrustX(mo, mo.angle+ANGLE_180 + ang, 24*mo.scale)
		local y = P_ReturnThrustY(mo, mo.angle+ANGLE_180 + ang, 24*mo.scale)
		local dust = P_SpawnMobjFromMobj(mo, x, y, 0, MT_THOK)
		dust.scale = 2*mo.scale/3
		dust.angle = R_PointToAngle2(dust.x, dust.y, mo.x, mo.y)
		dust.state = S_DROP_DUST
		dust.disappear = 8
		dust.tics = 8
		dust.momx = FixedMul(factor, cos(dust.angle - i * ANGLE_90))
		dust.momy = FixedMul(factor, sin(dust.angle - i * ANGLE_90))
	end
end)

rawset(_G, "DD_ThrustValue", function(p)
	local mo = p.mo
	if mo and p.speed >= p.runspeed
	and P_IsObjectOnGround(mo)
	and mo.z == mo.lastz then
		DROPTHRUST = (p.speed)+8*FRACUNIT
		return DROPTHRUST
	elseif mo and p.speed < p.runspeed
	and P_IsObjectOnGround(mo)
	and mo.z == mo.lastz then
		DROPTHRUST = 55*FRACUNIT
		return DROPTHRUST
	elseif mo and p.speed >= p.runspeed
	and P_IsObjectOnGround(mo)
	and mo.z > mo.lastz then
		DROPTHRUST = (p.speed)+10*FRACUNIT
		return DROPTHRUST
	elseif mo and p.speed >= p.runspeed
	and P_IsObjectOnGround(mo)
	and mo.z < mo.lastz then
		DROPTHRUST = (p.speed)+8*FRACUNIT
		return DROPTHRUST
	end
	if DROPTHRUST >= 56*FRACUNIT then
		DROPTHRUST = (DROPTHRUST/2)+18*FRACUNIT
	end
	if leveltime % 2 == 0 then
		mo.lastz = mo.z
	end
	return DROPTHRUST or 0
end)

-- The main code
addHook("SpinSpecial", function(p)
	local mo = p.mo
	if mo and mo.skin == DROPSKIN
	and not mo.dropdash
	and not mo.spindown
	and not P_IsObjectOnGround(mo)
	and mo.state == S_PLAY_JUMP then
		mo.dropdash = true
		p.charability = CA_NONE
	end
end)

addHook("PlayerThink", function(p)
	local mo = p.mo
	if mo and mo.state == S_PLAY_JUMP then
		p.pflags = $ & ~PF_SPINNING
	end
	if mo.dropdash then
		mo.droptime = $1+1
	end
	if mo.droptime == 10 then
		mo.droptime = $1+1
		mo.state = S_DROP_DASH
        S_StartSound(mo, sfx_drpdsh)
	end
	if mo and mo.dropdash
    and not mo.spindown then
		mo.droptime = 0
		mo.state = S_PLAY_JUMP
		p.charability = CA_THOK
		mo.dropdash = false
	end
	if mo and p.powers[pw_justsprung]
    and mo.momz > 0 then
		mo.droptime = 0
		mo.state = S_PLAY_SPRING
		mo.dropdash = false
	end
	if mo and P_PlayerInPain(p) then
		mo.droptime = 0
		mo.state = S_PLAY_PAIN
		mo.dropdash = false
	end
	if mo and P_CheckDeathPitCollide(mo) then
		mo.droptime = 0
		mo.state = S_PLAY_DEAD
		mo.dropdash = false
	end
	if mo and mo.skin == DROPSKIN
	and not p.powers[pw_super]
	and mo.dropdash and mo.droptime >= 10
	and P_IsObjectOnGround(mo) == true then
		P_InstaThrust(mo, mo.angle, DD_ThrustValue(p))
		DD_DoDust(p)
		S_StartSound(mo, sfx_zoom)
		mo.state = S_PLAY_ROLL
		p.charability = CA_THOK
		mo.camerascale = skins[mo.skin].camerascale*2
		mo.dropdash = false
		mo.droptime = 0
		p.pflags = $|PF_SPINNING
	end
	if mo and mo.skin == DROPSKIN
	and p.powers[pw_super]
	and mo.dropdash and mo.droptime >= 10
	and P_IsObjectOnGround(mo) == true then
		P_InstaThrust(mo, mo.angle, DD_ThrustValue(p))
		DD_DoDust(p)
		S_StartSound(mo, sfx_zoom)
		S_StartSound(mo, kc40)
		mo.state = S_PLAY_ROLL
		p.charability = CA_THOK
		mo.camerascale = skins[mo.skin].camerascale*2
		mo.dropdash = false
		mo.droptime = 0
		p.pflags = $|PF_SPINNING
	end
end)